package com.example.onomatopoeiaback.controller;


import com.example.onomatopoeiaback.domain.employee.Employee;
import com.example.onomatopoeiaback.domain.employee.EmployeeDTO;
import com.example.onomatopoeiaback.domain.visit.Visit;
import com.example.onomatopoeiaback.domain.visit.VisitDTO;
import com.example.onomatopoeiaback.security.Auth;
import com.example.onomatopoeiaback.service.EmployeeService;
import com.example.onomatopoeiaback.service.VisitService;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static com.example.onomatopoeiaback.domain.employee.CheckPosition.checkIsAdmin;

@RestController
@RequestMapping("/employee")
public class EmployeeController {
    private final VisitService visitService;
    private final EmployeeService employeeService;

    public EmployeeController(EmployeeService employeeService, VisitService visitService) {
        this.employeeService = employeeService;
        this.visitService = visitService;
    }

    @PutMapping("/ban/{login}")
    @SecurityRequirement(name = "basicAuth")
    public ResponseEntity<Employee> banEmployee(Authentication authentication, @PathVariable String login) {
        checkIsAdmin(authentication);
        employeeService.banUser(login);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @PutMapping("/unban/{login}")
    @SecurityRequirement(name = "basicAuth")
    public ResponseEntity<Employee> unbanEmployee(Authentication authentication, @PathVariable String login) {
        checkIsAdmin(authentication);
        employeeService.unbanUser(login);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @PostMapping("/create")
    @SecurityRequirement(name = "basicAuth")
    public ResponseEntity<Employee> createEmployee(Authentication authentication, @RequestBody EmployeeDTO employeeDTO) {
        checkIsAdmin(authentication);
        return ResponseEntity.ok(employeeService.createEmployee(employeeDTO));
    }

    @PatchMapping("/open")
    @SecurityRequirement(name = "basicAuth")
    public ResponseEntity<Visit> open(Authentication authentication, @RequestBody VisitDTO visitDTO) {
        visitService.register(Auth.getEmployee(authentication), visitDTO);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @GetMapping("/{username}/info")
    @SecurityRequirement(name = "basicAuth")
    public ResponseEntity<Employee> info(Authentication authentication, @PathVariable String username) {
        return ResponseEntity.ok(employeeService.info(Auth.getEmployee(authentication), username));
    }

    @GetMapping("/auth")
    @SecurityRequirement(name = "basicAuth")
    public ResponseEntity<Employee> auth() {
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @GetMapping("/search")
    @SecurityRequirement(name = "basicAuth")
    public ResponseEntity<Page<Employee>> searchUsers(
            Authentication authentication,
            @RequestParam String fullName,
            @RequestParam(defaultValue = "0") int page,
            @RequestParam(defaultValue = "10") int size
    ) {
        checkIsAdmin(authentication);
        Page<Employee> foundEmployees = employeeService.searchEmployee(fullName, page, size);
        return ResponseEntity.ok(foundEmployees);
    }
}
