package com.example.onomatopoeiaback.service;

import com.example.onomatopoeiaback.domain.employee.Employee;
import com.example.onomatopoeiaback.domain.employee.EmployeeDTO;
import com.example.onomatopoeiaback.domain.employee.PositionType;
import com.example.onomatopoeiaback.exceptions.ForbiddenException;
import com.example.onomatopoeiaback.exceptions.NotFoundException;
import com.example.onomatopoeiaback.repository.EmployeeRepository;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.awt.print.Pageable;
import java.util.Optional;

@Service
public class EmployeeService {

    private final EmployeeRepository employeeRepository;

    public EmployeeService(EmployeeRepository employeeRepository) {
        this.employeeRepository = employeeRepository;
    }

    public Employee createEmployee(EmployeeDTO employeeDTO) {
        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

        Employee employee = new Employee();
        employee.setLogin(employeeDTO.getLogin());
        employee.setPassword(passwordEncoder.encode(employeeDTO.getPassword()));
        employee.setFirstName(employeeDTO.getFirstName());
        employee.setLastName(employeeDTO.getLastName());
        employee.setPatronymic(employeeDTO.getPatronymic());
        employee.setPhoto(employeeDTO.getPhoto());
        employee.setPosition(employeeDTO.getPosition());

        employee = employeeRepository.saveAndFlush(employee);
        return employee;
    }

    public Employee info(Employee employee, String login) {
        Optional<Employee> requestedEmployeeOptional = employeeRepository.findByLogin(login);
        if (requestedEmployeeOptional.isEmpty()) {
            throw new NotFoundException();
        }
        Employee requestedEmployee = requestedEmployeeOptional.get();
        if (employee.getPosition().equals(PositionType.ADMINISTRATOR) || employee.getId() == requestedEmployee.getId()) {
            return requestedEmployee;
        }
        throw new ForbiddenException();
    }

    public Page<Employee> searchEmployee(String name, Integer page, Integer size) {
        PageRequest pageable = PageRequest.of(page, size);
        return employeeRepository.searchEmployees(name, pageable);
    }

    public void banUser(String login) {
        Optional<Employee> requestedEmployeeOptional = employeeRepository.findByLogin(login);
        if (requestedEmployeeOptional.isEmpty()) {
            throw new NotFoundException();
        }
        Employee requestedEmployee = requestedEmployeeOptional.get();
        requestedEmployee.setIsBanned(true);
        employeeRepository.saveAndFlush(requestedEmployee);
    }

    public void unbanUser(String login) {
        Optional<Employee> requestedEmployeeOptional = employeeRepository.findByLogin(login);
        if (requestedEmployeeOptional.isEmpty()) {
            throw new NotFoundException();
        }
        Employee requestedEmployee = requestedEmployeeOptional.get();
        requestedEmployee.setIsBanned(false);
        employeeRepository.saveAndFlush(requestedEmployee);
    }
}
